@@ -25,6 +25,9 @@
 
 #include <gio/gunixinputstream.h>
 
+#include <sys/types.h>
+#include <pwd.h>
+
 gboolean
 gitg_platform_support_use_native_window_controls (GdkDisplay *display)
 {
@@ -117,4 +120,34 @@ gitg_platform_support_get_data_dir (void)
 	return g_strdup (GITG_DATADIR);
 }
 
+gchar *
+gitg_platform_support_get_user_home_dir (const gchar *name)
+{
+	struct passwd *pwd;
+
+	if (name == NULL)
+	{
+		name = g_get_user_name ();
+	}
+
+	if (name == NULL)
+	{
+		return NULL;
+	}
+
+	pwd = getpwnam (name);
+
+	if (pwd == NULL)
+	{
+		return NULL;
+	}
+
+	return g_strdup (pwd->pw_dir);
+}
+
+void
+gitg_platform_support_application_support_prepare_startup (void)
+{
+}
+
 // ex:ts=4 noet
